#!/usr/bin/env python3

from PyQt5.QtCore import QUrl, Qt
from PyQt5.QtGui import QColor, QGuiApplication
from PyQt5.QtQml import QQmlComponent, qmlRegisterType
from PyQt5.QtQuick import QQuickItem, QSGGeometryNode, QSGGeometry, QSGFlatColorMaterial, QQuickView

import sys

class CustomItem(QQuickItem):
    def __init__(self, parent = None):
        super().__init__(parent)

        self.setFlag(QQuickItem.ItemHasContents)
        self.setAcceptedMouseButtons(Qt.LeftButton)
        self.setAcceptHoverEvents(True)

        self._corner_size = 15
        self._color = QColor(255, 0, 0, 255)

    def hoverEnterEvent(self, event):
        self._color = QColor(0, 255, 0, 255)
        self.update()

    def hoverLeaveEvent(self, event):
        self._color = QColor(255, 0, 0, 255)
        self.update()

    def updatePaintNode(self, paint_node, update_data):
        ### Does not Crash ###

        if paint_node is None:
            paint_node = QSGGeometryNode()

        geometry = QSGGeometry(QSGGeometry.defaultAttributes_Point2D(), 4, 6)
        geometry.setDrawingMode(QSGGeometry.GL_TRIANGLES)
        geometry.vertexDataAsPoint2D()[0].set(0, 0)
        geometry.vertexDataAsPoint2D()[1].set(0, self.height())
        geometry.vertexDataAsPoint2D()[2].set(self.width(), self.height())
        geometry.vertexDataAsPoint2D()[3].set(self.width(), 0)

        geometry.indexDataAsUShort()[0] = 0
        geometry.indexDataAsUShort()[1] = 1
        geometry.indexDataAsUShort()[2] = 3

        geometry.indexDataAsUShort()[3] = 1
        geometry.indexDataAsUShort()[4] = 2
        geometry.indexDataAsUShort()[5] = 3

        paint_node.setGeometry(geometry)

        material = QSGFlatColorMaterial()
        material.setColor(self._color)

        paint_node.setMaterial(material)

        # I need to store these otherwise they will be garbage collected before rendering.
        # With PyQt 5.4 this just means they never show up, with PyQt 5.5 it crashes.
        self._paint_node = paint_node
        self._geometry = geometry
        self._material = material

        return paint_node

        ### Crashes on the first call to update() ###

        #self._paint_node = QSGGeometryNode()

        #self._geometry = QSGGeometry(QSGGeometry.defaultAttributes_Point2D(), 4, 6)
        #self._geometry.setDrawingMode(QSGGeometry.GL_TRIANGLES)
        #self._geometry.vertexDataAsPoint2D()[0].set(0, 0)
        #self._geometry.vertexDataAsPoint2D()[1].set(0, self.height())
        #self._geometry.vertexDataAsPoint2D()[2].set(self.width(), self.height())
        #self._geometry.vertexDataAsPoint2D()[3].set(self.width(), 0)

        #self._geometry.indexDataAsUShort()[0] = 0
        #self._geometry.indexDataAsUShort()[1] = 1
        #self._geometry.indexDataAsUShort()[2] = 3

        #self._geometry.indexDataAsUShort()[3] = 1
        #self._geometry.indexDataAsUShort()[4] = 2
        #self._geometry.indexDataAsUShort()[5] = 3

        #self._paint_node.setGeometry(self._geometry)

        #self._material = QSGFlatColorMaterial()
        #self._material.setColor(self._color)

        #self._paint_node.setMaterial(self._material)

        #return self._paint_node

app = QGuiApplication(sys.argv)
view = QQuickView()
qmlRegisterType(CustomItem, "CustomItem", 1, 0, "CustomItem")
view.setSource(QUrl.fromLocalFile("./pyqt_bug.qml"))
view.show()
app.exec_()
